@@ -11,6 +11,9 @@ from .camera import EzvizCamera
 from .client import EzvizClient
 from .constants import DefenseModeType
 from .mqtt import MQTTClient
+from .exceptions import (
+    EzvizAuthVerificationCode,
+)
 
 
 def main() -> Any:
@@ -74,7 +77,9 @@ def main() -> Any:
         choices=range(1, 10),
     )
 
-    parser_camera_move_coords = subparsers_camera.add_parser("move_coords", help="Move the camera to the X,Y coordinates")
+    parser_camera_move_coords = subparsers_camera.add_parser(
+        "move_coords", help="Move the camera to the X,Y coordinates"
+    )
     parser_camera_move_coords.add_argument(
         "--x",
         required=True,
@@ -95,7 +100,15 @@ def main() -> Any:
         "--switch",
         required=True,
         help="Switch to switch",
-        choices=["audio", "ir", "state", "privacy", "sleep", "follow_move"],
+        choices=[
+            "audio",
+            "ir",
+            "state",
+            "privacy",
+            "sleep",
+            "follow_move",
+            "sound_alarm",
+        ],
     )
     parser_camera_switch.add_argument(
         "--enable",
@@ -126,6 +139,19 @@ def main() -> Any:
         type=int,
         choices=range(0, 100),
     )
+    parser_camera_alarm.add_argument(
+        "--do_not_disturb",
+        required=False,
+        help="\
+If alarm notifications are enabled in the EZViz app then movement normally causes a notification to be sent. \
+Enabling this feature stops these notifications, i.e. you are not to be disturbed even if movement occurs. \
+Care must be taken because do-not-disturb can not be reset using the mobile app. \
+No new notifications will be sent until do-not-disturb is disabled. \
+Movement is still recorded even if do-not-disturb is enabled.",
+        default=None,
+        type=int,
+        choices=[0, 1],
+    )
     parser_camera_alarm.add_argument(
         "--schedule", required=False, help="Schedule in json format *test*", type=str
     )
@@ -137,6 +163,15 @@ def main() -> Any:
     # print("--------------args")
 
     client = EzvizClient(args.username, args.password, args.region)
+    try:
+        client.login()
+
+    except EzvizAuthVerificationCode:
+        mfa_code = input("MFA code required, please input MFA code.\n")
+        client.login(sms_code=mfa_code)
+
+    except Exception as exp:  # pylint: disable=broad-except
+        print(exp)
 
     if args.debug:
         # You must initialize logging, otherwise you'll not see debug output.
@@ -150,7 +185,6 @@ def main() -> Any:
 
         if args.device_action == "device":
             try:
-                client.login()
                 print(json.dumps(client.get_device(), indent=2))
             except Exception as exp:  # pylint: disable=broad-except
                 print(exp)
@@ -159,7 +193,6 @@ def main() -> Any:
 
         elif args.device_action == "status":
             try:
-                client.login()
                 print(
                     pandas.DataFrame.from_dict(
                         data=client.load_cameras(),
@@ -200,7 +233,6 @@ def main() -> Any:
 
         elif args.device_action == "switch":
             try:
-                client.login()
                 print(json.dumps(client.get_switch(), indent=2))
             except Exception as exp:  # pylint: disable=broad-except
                 print(exp)
@@ -209,7 +241,6 @@ def main() -> Any:
 
         elif args.device_action == "connection":
             try:
-                client.login()
                 print(json.dumps(client.get_connection(), indent=2))
             except Exception as exp:  # pylint: disable=broad-except
                 print(exp)
@@ -223,7 +254,6 @@ def main() -> Any:
 
         if args.mode:
             try:
-                client.login()
                 print(
                     json.dumps(
                         client.api_set_defence_mode(
@@ -253,7 +283,6 @@ def main() -> Any:
 
         # load camera object
         try:
-            client.login()
             camera = EzvizCamera(client, args.serial)
             logging.debug("Camera loaded")
         except Exception as exp:  # pylint: disable=broad-except
@@ -300,6 +329,9 @@ def main() -> Any:
                     camera.switch_sleep_mode(args.enable)
                 elif args.switch == "follow_move":
                     camera.switch_follow_move(args.enable)
+                elif args.switch == "sound_alarm":
+                    # Map 0|1 enable flog to operation type: 1 for off and 2 for on.
+                    camera.switch_sound_alarm(args.enable + 1)
             except Exception as exp:  # pylint: disable=broad-except
                 print(exp)
             finally:
@@ -313,6 +345,8 @@ def main() -> Any:
                     camera.alarm_notify(args.notify)
                 if args.sensibility is not None:
                     camera.alarm_detection_sensibility(args.sensibility)
+                if args.do_not_disturb is not None:
+                    camera.do_not_disturb(args.do_not_disturb)
                 if args.schedule is not None:
                     camera.change_defence_schedule(args.schedule)
             except Exception as exp:  # pylint: disable=broad-except
