@@ -8,6 +8,32 @@ from uuid import uuid4
 
 import requests
 
+from .api_endpoints import (
+    API_ENDPOINT_ALARM_SOUND,
+    API_ENDPOINT_ALARMINFO_GET,
+    API_ENDPOINT_CAM_ENCRYPTKEY,
+    API_ENDPOINT_CANCEL_ALARM,
+    API_ENDPOINT_CREATE_PANORAMIC,
+    API_ENDPOINT_DETECTION_SENSIBILITY,
+    API_ENDPOINT_DETECTION_SENSIBILITY_GET,
+    API_ENDPOINT_DEVICES,
+    API_ENDPOINT_DO_NOT_DISTURB,
+    API_ENDPOINT_LOGIN,
+    API_ENDPOINT_LOGOUT,
+    API_ENDPOINT_PAGELIST,
+    API_ENDPOINT_PANORAMIC_DEVICES_OPERATION,
+    API_ENDPOINT_PTZCONTROL,
+    API_ENDPOINT_REFRESH_SESSION_ID,
+    API_ENDPOINT_RETURN_PANORAMIC,
+    API_ENDPOINT_SEND_CODE,
+    API_ENDPOINT_SERVER_INFO,
+    API_ENDPOINT_SET_DEFENCE_SCHEDULE,
+    API_ENDPOINT_SWITCH_DEFENCE_MODE,
+    API_ENDPOINT_SWITCH_SOUND_ALARM,
+    API_ENDPOINT_SWITCH_STATUS,
+    API_ENDPOINT_UPGRADE_DEVICE,
+    API_ENDPOINT_V3_ALARMS,
+)
 from .camera import EzvizCamera
 from .cas import EzvizCAS
 from .constants import (
@@ -28,26 +54,6 @@ from .exceptions import (
 
 _LOGGER = logging.getLogger(__name__)
 
-API_ENDPOINT_CLOUDDEVICES = "/api/cloud/v2/cloudDevices/getAll"
-API_ENDPOINT_PAGELIST = "/v3/userdevices/v1/resources/pagelist"
-API_ENDPOINT_DEVICES = "/v3/devices/"
-API_ENDPOINT_LOGIN = "/v3/users/login/v5"
-API_ENDPOINT_REFRESH_SESSION_ID = "/v3/apigateway/login"
-API_ENDPOINT_SWITCH_STATUS = "/switchStatus"
-API_ENDPOINT_PTZCONTROL = "/ptzControl"
-API_ENDPOINT_ALARM_SOUND = "/alarm/sound"
-API_ENDPOINT_DETECTION_SENSIBILITY = "/api/device/configAlgorithm"
-API_ENDPOINT_DETECTION_SENSIBILITY_GET = "/api/device/queryAlgorithmConfig"
-API_ENDPOINT_ALARMINFO_GET = "/v3/alarms/v2/advanced"
-API_ENDPOINT_SET_DEFENCE_SCHEDULE = "/api/device/defence/plan2"
-API_ENDPOINT_SWITCH_DEFENCE_MODE = "/v3/userdevices/v1/group/switchDefenceMode"
-API_ENDPOINT_SWITCH_SOUND_ALARM = "/sendAlarm"
-API_ENDPOINT_SERVER_INFO = "/v3/configurations/system/info"
-API_ENDPOINT_LOGOUT = "/v3/users/logout/v2"
-API_ENDPOINT_PANORAMIC_DEVICES_OPERATION = "/v3/panoramicDevices/operation"
-API_ENDPOINT_UPGRADE_DEVICE = "/v3/upgrades/v1/devices/"
-API_ENDPOINT_SEND_CODE = "/v3/sms/nologin/checkcode"
-
 
 class EzvizClient:
     """Initialize api client object."""
@@ -65,9 +71,9 @@ class EzvizClient:
         self.password = (
             hashlib.md5(password.encode("utf-8")).hexdigest() if password else None
         )  # Ezviz API sends md5 of password
-        self._session = None
-        self.close_session()
-        self._session.headers["sessionId"] = token.get("session_id") if token else None
+        self._session = requests.session()
+        self._session.headers.update(REQUEST_HEADER)
+        self._session.headers["sessionId"] = token["session_id"] if token else None
         self._token = token or {
             "session_id": None,
             "rf_session_id": None,
@@ -88,8 +94,9 @@ class EzvizClient:
             "account": self.account,
             "password": self.password,
             "featureCode": FEATURE_CODE,
-            "msgType": "0",
-            "cuName": "SFRDIDEw",
+            "msgType": "3" if smscode else "0",
+            "bizType": "TERMINAL_BIND" if smscode else "",
+            "cuName": "SGFzc2lv",  # hassio base64 encoded
             "smsCode": smscode,
         }
 
@@ -141,6 +148,9 @@ class EzvizClient:
             _LOGGER.warning("Your region url: %s", self._token["api_url"])
             return self.login()
 
+        if json_result["meta"]["code"] == 1012:
+            raise PyEzvizError("The MFA code is invalid, please try again.")
+
         if json_result["meta"]["code"] == 1013:
             raise PyEzvizError("Incorrect Username.")
 
@@ -186,7 +196,7 @@ class EzvizClient:
                 + str(req.text)
             ) from err
 
-        if json_output.get("meta").get("code") != 200:
+        if json_output["meta"].get("code") != 200:
             raise PyEzvizError(
                 f"Could not request MFA code: Got {req.status_code} : {req.text})"
             )
@@ -226,7 +236,7 @@ class EzvizClient:
                 + str(req.text)
             ) from err
 
-        if json_output.get("meta").get("code") != 200:
+        if json_output["meta"].get("code") != 200:
             raise PyEzvizError(f"Error getting Service URLs: {json_output['meta']}")
 
         service_urls = json_output["systemConfigInfo"]
@@ -276,7 +286,7 @@ class EzvizClient:
                 + str(req.text)
             ) from err
 
-        if json_output.get("meta").get("code") != 200:
+        if json_output["meta"].get("code") != 200:
             # session is wrong, need to relogin
             self.login()
             logging.info(
@@ -300,7 +310,7 @@ class EzvizClient:
 
         return json_result
 
-    def get_alarminfo(self, serial: str, max_retries: int = 0) -> Any:
+    def get_alarminfo(self, serial: str, limit: int = 1, max_retries: int = 0) -> Any:
         """Get data from alarm info API."""
         if max_retries > MAX_RETRIES:
             raise PyEzvizError("Can't gather proper data. Max retries exceeded.")
@@ -308,7 +318,7 @@ class EzvizClient:
         params: dict[str, int | str] = {
             "deviceSerials": serial,
             "queryType": -1,
-            "limit": 1,
+            "limit": limit,
             "stype": -1,
         }
 
@@ -325,7 +335,7 @@ class EzvizClient:
             if err.response.status_code == 401:
                 # session is wrong, need to relogin
                 self.login()
-                return self.get_alarminfo(serial, max_retries + 1)
+                return self.get_alarminfo(serial, limit, max_retries + 1)
 
             raise HTTPError from err
 
@@ -391,7 +401,7 @@ class EzvizClient:
                 + str(req.text)
             ) from err
 
-        if json_output.get("meta").get("code") != 200:
+        if json_output["meta"].get("code") != 200:
             raise PyEzvizError(
                 f"Could not set the switch: Got {req.status_code} : {req.text})"
             )
@@ -434,7 +444,7 @@ class EzvizClient:
                 + str(req.text)
             ) from err
 
-        if json_output.get("meta").get("code") != 200:
+        if json_output["meta"].get("code") != 200:
             raise PyEzvizError(
                 f"Could not initiate firmare upgrade: Got {req.status_code} : {req.text})"
             )
@@ -481,13 +491,54 @@ class EzvizClient:
                 + str(req.text)
             ) from err
 
-        if json_output.get("meta").get("code") != 200:
+        if json_output["meta"].get("code") != 200:
             raise PyEzvizError(
                 f"Could not set the alarm sound: Got {req.status_code} : {req.text})"
             )
 
         return True
 
+    # Not tested
+    def cancel_alarm_device(self, serial: str, max_retries: int = 0) -> bool:
+        """Cacnel alarm on an Alarm device."""
+        if max_retries > MAX_RETRIES:
+            raise PyEzvizError("Can't gather proper data. Max retries exceeded.")
+
+        try:
+            req = self._session.post(
+                "https://" + self._token["api_url"] + API_ENDPOINT_CANCEL_ALARM,
+                data={"subSerial": serial},
+                timeout=self._timeout,
+            )
+
+            req.raise_for_status()
+
+        except requests.HTTPError as err:
+            if err.response.status_code == 401:
+                # session is wrong, need to relogin
+                self.login()
+                return self.sound_alarm(serial, max_retries + 1)
+
+            raise HTTPError from err
+
+        try:
+            json_output = req.json()
+
+        except ValueError as err:
+            raise PyEzvizError(
+                "Impossible to decode response: "
+                + str(err)
+                + "\nResponse was: "
+                + str(req.text)
+            ) from err
+
+        if json_output["meta"].get("code") != 200:
+            raise PyEzvizError(
+                f"Could not cancel alarm siren: Got {req.status_code} : {req.text})"
+            )
+
+        return True
+
     def load_cameras(self) -> dict[Any, Any]:
         """Load and return all cameras objects."""
 
@@ -498,6 +549,7 @@ class EzvizClient:
             DeviceCatagories.BATTERY_CAMERA_DEVICE_CATEGORY.value,
             DeviceCatagories.DOORBELL_DEVICE_CATEGORY.value,
             DeviceCatagories.BASE_STATION_DEVICE_CATEGORY.value,
+            DeviceCatagories.CAT_EYE_CATEGORY.value,
         ]
 
         for device, data in devices.items():
@@ -599,6 +651,144 @@ class EzvizClient:
 
         return req.text
 
+    def get_cam_key(
+        self, serial: str, smscode: int | None = None, max_retries: int = 0
+    ) -> Any:
+        """Get Camera encryption key."""
+
+        if max_retries > MAX_RETRIES:
+            raise PyEzvizError("Can't gather proper data. Max retries exceeded.")
+
+        try:
+            req = self._session.post(
+                "https://" + self._token["api_url"] + API_ENDPOINT_CAM_ENCRYPTKEY,
+                data={
+                    "checkcode": smscode,
+                    "serial": serial,
+                    "clientNo": "web_site",
+                    "clientType": 3,
+                    "netType": "WIFI",
+                    "featureCode": FEATURE_CODE,
+                    "sessionId": self._token["session_id"],
+                },
+                timeout=self._timeout,
+            )
+
+            req.raise_for_status()
+
+        except requests.HTTPError as err:
+            if err.response.status_code == 401:
+                # session is wrong, need to relogin
+                self.login()
+                return self.get_cam_key(serial, smscode, max_retries + 1)
+
+            raise HTTPError from err
+
+        try:
+            json_output = req.json()
+
+        except ValueError as err:
+            raise PyEzvizError(
+                "Impossible to decode response: "
+                + str(err)
+                + "\nResponse was: "
+                + str(req.text)
+            ) from err
+
+        if json_output.get("resultCode") == "20002":
+            raise PyEzvizError(
+                f"MFA code required: Got {req.status_code} : {req.text})"
+            )
+
+        if json_output.get("resultCode") != "0":
+            raise PyEzvizError(
+                f"Could not get camera encryption key: Got {req.status_code} : {req.text})"
+            )
+
+        return json_output
+
+    def create_panoramic(self, serial: str, max_retries: int = 0) -> Any:
+        """Create panoramic image."""
+
+        if max_retries > MAX_RETRIES:
+            raise PyEzvizError("Can't gather proper data. Max retries exceeded.")
+
+        try:
+            req = self._session.post(
+                "https://" + self._token["api_url"] + API_ENDPOINT_CREATE_PANORAMIC,
+                data={"deviceSerial": serial},
+                timeout=self._timeout,
+            )
+
+            req.raise_for_status()
+
+        except requests.HTTPError as err:
+            if err.response.status_code == 401:
+                # session is wrong, need to relogin
+                self.login()
+                return self.create_panoramic(serial, max_retries + 1)
+
+            raise HTTPError from err
+
+        try:
+            json_output = req.json()
+
+        except ValueError as err:
+            raise PyEzvizError(
+                "Impossible to decode response: "
+                + str(err)
+                + "\nResponse was: "
+                + str(req.text)
+            ) from err
+
+        if json_output.get("resultCode") != "0":
+            raise PyEzvizError(
+                f"Could not send command to create panoramic photo: Got {req.status_code} : {req.text})"
+            )
+
+        return json_output
+
+    def return_panoramic(self, serial: str, max_retries: int = 0) -> Any:
+        """Return panoramic image url list."""
+
+        if max_retries > MAX_RETRIES:
+            raise PyEzvizError("Can't gather proper data. Max retries exceeded.")
+
+        try:
+            req = self._session.post(
+                "https://" + self._token["api_url"] + API_ENDPOINT_RETURN_PANORAMIC,
+                data={"deviceSerial": serial},
+                timeout=self._timeout,
+            )
+
+            req.raise_for_status()
+
+        except requests.HTTPError as err:
+            if err.response.status_code == 401:
+                # session is wrong, need to relogin
+                self.login()
+                return self.return_panoramic(serial, max_retries + 1)
+
+            raise HTTPError from err
+
+        try:
+            json_output = req.json()
+
+        except ValueError as err:
+            raise PyEzvizError(
+                "Impossible to decode response: "
+                + str(err)
+                + "\nResponse was: "
+                + str(req.text)
+            ) from err
+
+        if json_output.get("resultCode") != "0":
+            raise PyEzvizError(
+                f"Could retrieve panoramic photo: Got {req.status_code} : {req.text})"
+            )
+
+        return json_output
+
     def ptz_control_coordinates(
         self, serial: str, x_axis: float, y_axis: float
     ) -> bool:
@@ -827,7 +1017,53 @@ class EzvizClient:
                 + str(req.text)
             ) from err
 
-        if json_output.get("meta").get("code") != 200:
+        if json_output["meta"].get("code") != 200:
+            raise PyEzvizError(
+                f"Could not set defence mode: Got {req.status_code} : {req.text})"
+            )
+
+        return True
+
+    def do_not_disturb(
+        self,
+        serial: str,
+        enable: int = 1,
+        channelno: int = 1,
+        max_retries: int = 0,
+    ) -> bool | str:
+        """Set do not disturb on camera with spesified serial."""
+        if max_retries > MAX_RETRIES:
+            raise PyEzvizError("Can't gather proper data. Max retries exceeded.")
+
+        try:
+            req = self._session.put(
+                "https://"
+                + self._token["api_url"]
+                + API_ENDPOINT_V3_ALARMS
+                + serial
+                + "/"
+                + channelno
+                + API_ENDPOINT_DO_NOT_DISTURB,
+                data={"enable": enable, "channelNo": channelno, "deviceSerial": serial},
+                timeout=self._timeout,
+            )
+            req.raise_for_status()
+
+        except requests.HTTPError as err:
+            if err.response.status_code == 401:
+                # session is wrong, need to re-log-in
+                self.login()
+                return self.do_not_disturb(serial, enable, max_retries + 1)
+
+            raise HTTPError from err
+
+        try:
+            json_output = req.json()
+
+        except ValueError as err:
+            raise PyEzvizError("Could not decode response:" + str(err)) from err
+
+        if json_output["meta"].get("code") != 200:
             raise PyEzvizError(
                 f"Could not set defence mode: Got {req.status_code} : {req.text})"
             )
